var gridCont = Ext.getCmp('{{ component.south_region.client_id }}');
var displayContentField = Ext.getCmp('{{ component.display_content.client_id }}');
var signBtn = Ext.getCmp('{{ component.sign_btn.client_id }}');
var cancelBtn = Ext.getCmp('{{ component.cancel_btn.client_id }}');
var options =  Ext.decode('{{ component.options|safe|escapejs }}');

var certsGrid = new Ext.grid.GridPanel({
    store: new Ext.data.ArrayStore({
        fields: [
           {name: 'subjectName', mapping: 'subjectName'},
           {name: 'issuerName', mapping: 'issuerName'},
           {name: 'validFromDate', mapping: 'validFromDate'},
           {name: 'validToDate', mapping: 'validToDate'},
           {name: 'thumbprint', mapping: 'thumbprint'},
           {name: 'pubKeyAlgorithm', mapping: 'pubKeyAlgorithm'}
        ]
    }),
    columns: [
        {
            id       :'subjectName',
            header   : 'Владелец',
            width    : 150,
            sortable : true,
            dataIndex: 'subjectName'
        },
        {
            header   : 'Издатель',
            width    : 150,
            sortable : true,
            dataIndex: 'issuerName'
        },
        {
            header   : 'Выдан',
            width    : 120,
            sortable : true,
            dataIndex: 'validFromDate'
        },
        {
            header   : 'Действителен до',
            width    : 120,
            sortable : true,
            dataIndex: 'validToDate'
        },
        {
            header   : 'Алгоритм ключа',
            width    : 100,
            sortable : true,
            dataIndex: 'pubKeyAlgorithm'
        }
    ],
    title: 'Сертификат подписи'
});

gridCont.items.add(certsGrid);

var sinopsSigner = new Ext.m3.SinopsSigner({win: win});
win.sinopsSigner = sinopsSigner;
win.operationProcessMask = new Ext.LoadMask(win.getEl(), {msg: 'Выполнение операции'});

function loadCertificatesData(){
    var certs = CPro.getCertsList(true, new Date(), true);
    var data = [];
    Ext.each(certs, function(cert){
        var co = new CPro.Cert(cert);
        data.push({
            subjectName: co.getSubjectName(),
            issuerName: co.getIssuerName(),
            validFromDate: co.getCertFromDate(),
            validToDate: co.getCertTillDate(),
            pubKeyAlgorithm: co.getPubKeyAlgorithm(),
            thumbprint: cert.Thumbprint,
        });
    });
    certsGrid.getStore().loadData(data);
}

win.on('show', function(){
    sinopsSigner.requirePlugin(
        function(){
            loadCertificatesData();
            sinopsSigner.on('aftersign', operationSubmit);
        },
        function(){
            win.close(true);
        }
    );
});

signBtn.on('click', function(){
    var selectedCert,
        certCount = certsGrid.store.getCount(),
        sm = certsGrid.getSelectionModel();
    if(certCount==1){
        selectedCert = certsGrid.store.getAt(0);
    }else{
        if(sm.hasSelection()){
            selectedCert = sm.getSelected();
        }else{
            Ext.Msg.alert('Внимание!', 'Выберите сертификат подписи!');
            return;
        }
    }

    var signOptions = {
        signatureType: sinopsSigner.SIGNATURE_CODES_MAP[options.allowed_signatures[0]],
        detached: false,
        TSAAddress: options.tsa_address
    };
    sinopsSigner.sign(selectedCert.data.thumbprint, options.content_to_sign, signOptions);
});

cancelBtn.on('click', function(){
    win.close(true);
});


function defaultOperationPostAction(res, opt){
    function getTopWindow(exclude){
        exclude = exclude || [];
        var winds = [];
        Ext.each(Ext.query('.x-window'), function(obj){
            var w = Ext.getCmp(Ext.get(obj).id);
            if(exclude.indexOf(w)<0 && w.el.visible){
                winds.push(w);
            }
        });
        if(winds.length){
            winds.sort(function(a,b){return a.el.zindex - b.el.zindex;});
            return winds[winds.length-1];
        }
    }
    var parentWin  = win.parentWindow || Ext.getCmp(options.request_params.m3_window_id) || getTopWindow([win]);
    if(parentWin){
        parentWin.fireEvent('closed_ok', res.responseText);
        parentWin.close(true);
    }
    smart_eval(res.responseText);
}


win.setOperationPostAction = function(fn){
    win._operationPostAction = fn;
};

win.getOperationPostAction = function(){
    if(win._operationPostAction !== undefined){
        return win._operationPostAction;
    }
    return defaultOperationPostAction;
};


function operationSubmit(){
    var params = sinopsSigner.getSignatureParams();
    Ext.applyIf(params, options.request_params);

    var req = {
        url: options.request_url,
        method: 'POST',
        params: params,
        success: function(res, opt) {
            if(win.fireEvent('afteroperationsubmit', sinopsSigner, res, opt)){
                var data = Ext.decode(res.responseText);
                if(data && data.success){
                    sinopsSigner.showSuccessMsg({
                        fn: function(){
                            win.getOperationPostAction()(res, opt);
                        }
                    });
                }else{
                    smart_eval(res.responseText);
                }
            }
        },
        failure: uiAjaxFailMessage,
        callback: function(){
            win.close(true);
            win.operationProcessMask.hide();
        }
    };

    if(win.fireEvent('beforeoperationsubmit', sinopsSigner, req)){
        win.operationProcessMask.show();
        Ext.Ajax.request(req);
    }
}
